'use strict';

var browser = {}, Port, _;

if (document instanceof window.HTMLDocument) {
	browser = document.documentElement || document.createElementNS('http://www.w3.org/1999/xhtml', 'div');

	if (!window.requestAnimationFrame) {
		window.requestAnimationFrame = function(cb) {
			return window.setTimeout(cb, 25);
		};
	}

	if (browser.style) {
		_ = browser.style;

		browser = {
			'wheel': 'onwheel' in browser ? 'wheel' : 'mousewheel',
			'transform': ('webkitTransform' in _ ? 'webkitT' : 't') + 'ransform',
			'transform_css': ('webkitTransform' in _ ? '-webkit-' : '') + 'transform',
			'transition_css': ('webkitTransition' in _ ? '-webkit-' : '') + 'transition',
			'transitionend': 'webkitTransition' in _ ? 'webkitTransitionEnd' : 'transitionend',
			'box-sizing_css': ('MozBoxSizing' in _ ? '-moz-' : ('WebkitBoxSizing' in _ ? '-webkit-' : '')) + 'box-sizing',
			'zoom-in': this.chrome || this.mx || this.safari ? '-webkit-zoom-in' : 'zoom-in'
		};
		_ = null;
	}
}

function buildNodes(host, nodes) {
	if (!host || !Array.isArray(nodes)) {
		return;
	}

	if (!nodes.length) {
		return host;
	}

	var doc = host.ownerDocument;
	var fragment = doc.createDocumentFragment();

	for (var i = 0, l = nodes.length; i < l; ++i) {
		if (!nodes[i]) {
			continue;
		}

		if (typeof nodes[i] === 'string') {
			fragment.appendChild(doc.createTextNode(nodes[i]));
			continue;
		}

		var node = doc.createElement(nodes[i].tag);

		if (nodes[i].attrs) {
			for (var attr in nodes[i].attrs) {
				// bypass CSP
				if (attr === 'style') {
					node.style.cssText = nodes[i].attrs[attr];
				}
				else {
					node.setAttribute(attr, nodes[i].attrs[attr]);
				}
			}
		}

		if (nodes[i].nodes) {
			buildNodes(node, nodes[i].nodes);
		}
		else if (nodes[i].text) {
			node.textContent = nodes[i].text;
		}

		fragment.appendChild(node);
	}

	if (fragment.childNodes.length) {
		host.appendChild(fragment);
	}

	return host;
}

function mediaType(type) {
	if (type) {
		document.mediaType = type;
	}

	if (document.mediaType) {
		return document.mediaType;
	}

	var d = document, img = d.images && d.images.length === 1 && d.images[0];

	if (img && img.parentNode === d.body && img.src === window.location.href) {
		return document.mediaType = 'img';
	}

	d = d.querySelector(
		'body > video:first-child, body > div:only-child > video:first-child'
	);

	// Firefox doesn't handle redirects properly, so it gives the previous URL here
	if (d && (browser.firefox || d.currentSrc === window.location.href)) {
		return document.mediaType = 'video';
	}
}

browser.opera = true;

var LNG;

Port = {
	listen: function(listener) {
		if (this.listener) {
			opera.extension.removeEventListener('message', this.listener, false);
		}

		if (typeof listener === 'function') {
			this.listener = function(e) {
				listener(e.data);
			};
			opera.extension.addEventListener('message', this.listener, false);
		}
		else {
			this.listener = null;
		}
	},
	send: function(msg) {
		opera.extension.postMessage(msg);
	}
};

if (window.location.protocol === 'widget:') {
	_ = location.pathname;
	LNG = document.createElement('script');
	LNG.src = './' + _.slice(1, _.indexOf('.')) + '_lng.js';
	document.head.appendChild(LNG);

	_ = function(s) {
		return typeof LNG === 'object' && LNG[s] || s;
	};

	browser.insertHTML = function(n, html) { n.innerHTML = html; };
}
