/* global browser, Port, mediaType */
"use strict";

// Should not run on image pages
if (!mediaType() && document instanceof window.HTMLDocument
	&& (!browser.safari || location.protocol !== 'safari-extension:')) {
	Port.listen(function(cfg) {
		Port.listen(null);

		cfg = cfg && cfg.prefs && JSON.parse(cfg.prefs).opener;

		if (!cfg) {
			return;
		}

		var mdownstart, md_x, md_y;
		var checkBG = function(imgs) {
			// ("...\"") - Gecko
			// (...) or ('...)') - WebKit
			// ("...&quot;") - Presto
			var rgx = /\burl\(([^'"\)][^\)]*|"[^"\\]+(?:\\.[^"\\]*)*|'[^'\\]+(?:\\.[^'\\]*)*)(?=['"]?\))/g;
			imgs = (imgs.backgroundImage + imgs.content + imgs.listStyleImage).match(rgx);

			if (imgs === null) {
				return imgs;
			}

			var i, pos;

			for (i = 0; i < imgs.length; ++i) {
				pos = imgs[i][4];
				pos = pos === '"' || pos === "'" ? 5 : 4;
				imgs[i] = imgs[i].slice(pos);
			}

			return imgs;
		};
		var checkIMG = function(node) {
			var nname = node.nodeName.toUpperCase();

			if (nname === 'IMG' || nname === 'EMBED' || node.type === 'image') {
				return node.src;
			}
			else if (nname === 'CANVAS') {
				return node.toDataURL();
			}
			else if (nname === 'OBJECT') {
				if (node.data) {
					return node.data;
				}
			}
			else if (nname === 'AREA') {
				node = document.querySelector('img[usemap="#' + node.parentNode.name + '"]');

				if (node && node.src) {
					return node.src;
				}
			}
			else if (nname === 'VIDEO') {
				nname = document.createElement('canvas');
				nname.width = node.clientWidth;
				nname.height = node.clientHeight;
				nname.getContext('2d').drawImage(node, 0, 0, nname.width, nname.height);
				return nname.toDataURL('image/jpeg');
			}
			else if (/^\[object SVG/.test(node.toString())) {
				node = (new window.XMLSerializer).serializeToString(
					node.ownerSVGElement === null ? node : node.ownerSVGElement
				);

				if (typeof node === 'string') {
					return 'data:image/svg+xml,' + encodeURIComponent(node);
				}
			}
			else if (node.poster) {
				return node.poster;
			}

			return null;
		};

		window.addEventListener('mousedown', function(e) {
			if (e.button !== 2) {
				return;
			}

			mdownstart = e.timeStamp;
			// context menu should work where the mousedown event happened (Chrome...)
			md_x = e.clientX;
			md_y = e.clientY;
		}, true);

		window.addEventListener('contextmenu', function(e) {
			if (!mdownstart || e.button !== 2
				|| md_x != e.clientX || md_y != e.clientY) {
				return;
			}

			var elapsed = e.timeStamp - mdownstart >= 300;

			if (!(e.ctrlKey && e.altKey && !e.shiftKey && !elapsed && cfg > 1
					|| elapsed && (cfg === 1 || cfg === 3) && !e.ctrlKey && !e.shiftKey && !e.altKey)) {
				return;
			}

			var url, r;
			var urls = []
			var el = 30;
			var reIgnore = /^(html|body)$/i;
			// not exactly what we want
			var xpath = document.evaluate(
				'. | ./ancestor::*[position()<' + el
				+ '] | ./preceding::*[position()<' + el
				+ '][not(head) and not(ancestor::head)] | ./descendant::*[position()<'
				+ el + '] | ./following::*[position()<' + el + ']',
				e.target, null, 4, null
			);

			while (el = xpath.iterateNext()) {
				if (reIgnore.test(el.nodeName) && !reIgnore.test(e.target.nodeName)) {
					continue;
				}

				r = el.getBoundingClientRect();

				if (!r || !(e.clientX >= r.left && e.clientX <= r.left + r.width
					&& e.clientY >= r.top && e.clientY <= r.top + r.height)) {
					continue;
				}

				if (url = checkIMG(el)) {
					urls.push(url);
				}

				if (url = checkBG(window.getComputedStyle(el))) {
					urls = urls.concat(url);
				}
			}

			if (!urls || !urls.length) {
				return;
			}

			e.stopImmediatePropagation();
			e.preventDefault();

			var i;
			var filter = {};

			for (i = 0; i < urls.length; ++i) {
				filter[urls[i]] = true;
			}

			Port.send({
				cmd: 'open',
				url: Object.keys(filter).reverse()
			});
		}, true);
	});

	Port.send({cmd: 'hello'});
}
